class CuttingPlan {
    constructor() {
        this.sheetWidth = {{SHEET_WIDTH}};
        this.sheetHeight = {{SHEET_HEIGHT}};
        this.sawKerf = 4.0; // Adicionando sawKerf que estava faltando
        this.pieces = {};
        this.sheets = {};
        this.scale = 0.35;

        this.initializeEventListeners();
    }



    initializeEventListeners() {
        document.getElementById('calculate').addEventListener('click', () => this.generateCuttingPlan());
       // document.getElementById('downloadPdf').addEventListener('click', () => this.generatePDF());
        document.getElementById('downloadCsv').addEventListener('click', () => this.exportToCSV());
        document.getElementById('selectAll').addEventListener('change', (e) => this.toggleAllPieces(e));

        document.querySelectorAll('.piece-selector').forEach(checkbox => {
            checkbox.addEventListener('change', () => this.updatePieceSelection());
        });
        // Atualizar o listener do botão PDF
        const pdfButton = document.getElementById('downloadPdf');
        if (pdfButton) {
            pdfButton.addEventListener('click', () => {
                if (this.sheets && Object.keys(this.sheets).length > 0) {
                    this.generatePDF();
                } else {
                    alert('Por favor, gere um plano de corte primeiro.');
                }
            });
        }

        // Habilitar o botão PDF quando houver um plano gerado
        document.getElementById('calculate').addEventListener('click', () => {
            const pdfButton = document.getElementById('downloadPdf');
            if (pdfButton) {
                pdfButton.disabled = false;
            }
        });
    }


    collectPieces() {
        this.pieces = {};
        const rows = document.querySelectorAll('#componentsList tbody tr:not(.material-header)');

        rows.forEach(row => {
            const checkbox = row.querySelector('.piece-selector');
            if (!checkbox?.checked) return;

            const material = row.dataset.material || 'default';
            if (!this.pieces[material]) {
                this.pieces[material] = [];
            }

            try {
                const dimensions = JSON.parse(row.dataset.dimensions);
                const name = row.querySelector('td:nth-child(2)').textContent;
                const quantity = parseInt(row.querySelector('td:nth-child(4)').textContent);
                const canRotate = !document.getElementById('respectGrain')?.checked ||
                row.querySelector('.piece-rotation')?.checked;

                if (Array.isArray(dimensions) && dimensions.length >= 2) {
                    // Ordena as dimensões para que a maior seja sempre a largura
                    const [width, height] = dimensions.sort((a, b) => b - a);

                    for (let i = 0; i < quantity; i++) {
                        this.pieces[material].push({
                            name: `${name} ${i + 1}`,
                            width: parseFloat(width),
                                                   height: parseFloat(height),
                                                   canRotate: canRotate,
                                                   id: `${name}-${i}`
                        });
                    }
                }
            } catch (e) {
                console.error('Erro ao processar peça:', e);
            }
        });
    }

    calculateBudget() {
        let total = 0;

        // Calcular custos dos materiais
        const materialRows = document.querySelectorAll('#materialSummary tbody tr');
        materialRows.forEach(row => {
            const quantity = parseFloat(row.querySelector('.quantity')?.value) || 0;
            const unitCost = parseFloat(row.querySelector('.unit-cost')?.value) || 0;
            const totalCell = row.querySelector('.total-cost');

            const rowTotal = quantity * unitCost;
            if (totalCell) {
                totalCell.textContent = `R$ ${rowTotal.toFixed(2)}`;
            }
            total += rowTotal;
        });

        // Calcular custos de mão de obra
        const laborRows = document.querySelectorAll('#laborSummary tbody tr');
        laborRows.forEach(row => {
            const hours = parseFloat(row.querySelector('.hours')?.value) || 0;
            const hourlyRate = parseFloat(row.querySelector('.hourly-rate')?.value) || 0;
            const totalCell = row.querySelector('.total-cost');

            const rowTotal = hours * hourlyRate;
            if (totalCell) {
                totalCell.textContent = `R$ ${rowTotal.toFixed(2)}`;
            }
            total += rowTotal;
        });

        // Calcular outros custos
        const otherCostsRows = document.querySelectorAll('#otherCostsSummary tbody tr');
        otherCostsRows.forEach(row => {
            const cost = parseFloat(row.querySelector('.cost')?.value) || 0;
            total += cost;
        });

        // Atualizar o total geral
        const grandTotalElement = document.getElementById('grandTotal');
        if (grandTotalElement) {
            grandTotalElement.textContent = `R$ ${total.toFixed(2)}`;
        }

        return total;
    }

    // Adicione esta função para configurar os eventos do orçamento
    setupBudgetEvents() {
        // Adicionar evento ao botão de cálculo
        const calculateButton = document.getElementById('calculateBudget');
        if (calculateButton) {
            calculateButton.addEventListener('click', () => this.calculateBudget());
        }

        // Atualizar totais quando valores são alterados
        const inputs = document.querySelectorAll('#materialSummary input, #laborSummary input, #otherCostsSummary input');
        inputs.forEach(input => {
            input.addEventListener('input', () => {
                const row = input.closest('tr');
                if (row) {
                    this.updateRowTotal(row);
                }
            });
        });
    }

    // Função auxiliar para atualizar o total de uma linha
    updateRowTotal(row) {
        const totalCell = row.querySelector('.total-cost');
        if (!totalCell) return;

        let rowTotal = 0;
        const quantity = parseFloat(row.querySelector('.quantity')?.value) || 0;
        const unitCost = parseFloat(row.querySelector('.unit-cost')?.value) || 0;
        const hours = parseFloat(row.querySelector('.hours')?.value) || 0;
        const hourlyRate = parseFloat(row.querySelector('.hourly-rate')?.value) || 0;

        if (quantity && unitCost) {
            rowTotal = quantity * unitCost;
        } else if (hours && hourlyRate) {
            rowTotal = hours * hourlyRate;
        }

        totalCell.textContent = `R$ ${rowTotal.toFixed(2)}`;
        this.calculateBudget(); // Recalcula o total geral
    }






    generateCuttingPlan() {
        this.collectPieces();
        this.sheets = {};
        const margin = parseInt(document.getElementById('margin')?.value) || 4;
        const allowRotation = document.getElementById('allowRotation')?.checked ?? true;

        // Gera o plano de corte
        Object.entries(this.pieces).forEach(([material, pieces]) => {
            if (!pieces || !pieces.length) return;

            this.sheets[material] = [];
            // Ordena as peças pela altura (em vez de área) para otimizar o empilhamento vertical
            pieces.sort((a, b) => b.height - a.height);

            pieces.forEach(piece => {
                let placed = false;

                // Tenta colocar na última chapa primeiro
                if (this.sheets[material].length > 0) {
                    const lastSheet = this.sheets[material][this.sheets[material].length - 1];
                    placed = this.tryPlacePiece(lastSheet, piece, margin, allowRotation);
                }

                // Se não conseguiu colocar na última chapa, tenta nas outras
                if (!placed) {
                    for (let i = 0; i < this.sheets[material].length && !placed; i++) {
                        placed = this.tryPlacePiece(this.sheets[material][i], piece, margin, allowRotation);
                    }
                }

                // Se ainda não conseguiu colocar, cria uma nova chapa
                if (!placed) {
                    const newSheet = this.createNewSheet();
                    this.sheets[material].push(newSheet);
                    this.tryPlacePiece(newSheet, piece, margin, allowRotation);
                }
            });
        });

        // Atualiza a quantidade de chapas na tabela de orçamento
        Object.entries(this.sheets).forEach(([material, sheets]) => {
            const materialRow = document.querySelector(`#materialSummary tr[data-itemcode="${material}"]`);
            if (materialRow) {
                const quantityInput = materialRow.querySelector('.quantity');
                if (quantityInput) {
                    quantityInput.value = sheets.length;
                    // Atualiza o total
                    const unitCost = parseFloat(materialRow.querySelector('.unit-cost').value) || 0;
                    const totalCost = sheets.length * unitCost;
                    materialRow.querySelector('.total-cost').textContent = `R$ ${totalCost.toFixed(2)}`;
                }
            }
        });





        this.renderCuttingPlan();
        this.updateStatistics();
        // Recalcula o orçamento total
        calculateBudget();
    }





    createNewSheet() {
        return {
            width: this.sheetWidth,
            height: this.sheetHeight,
            spaces: [{
                x: 0,
                y: 0,
                width: this.sheetWidth,
                height: this.sheetHeight
            }],
            placements: []
        };
    }

    tryPlacePiece(sheet, piece, margin, allowRotation) {
        let bestFit = null;
        let bestSpace = null;
        let bestRotation = false;

        // Ordena os espaços disponíveis de cima para baixo
        const sortedSpaces = [...sheet.spaces].sort((a, b) => a.y - b.y);

        for (const space of sortedSpaces) {
            const pieceWidth = piece.width + margin * 2 + this.sawKerf;
            const pieceHeight = piece.height + margin * 2 + this.sawKerf;

            // Tenta primeiro com a maior dimensão na horizontal
            if (this.fitsInSpace(space, pieceWidth, pieceHeight)) {
                bestFit = { x: space.x, y: space.y };
                bestSpace = space;
                bestRotation = false;
                break;  // Use o primeiro espaço adequado encontrado
            }

            // Se permitido rotação e a peça puder ser rotacionada, tenta na vertical
            if (allowRotation && piece.canRotate &&
                this.fitsInSpace(space, pieceHeight, pieceWidth)) {
                bestFit = { x: space.x, y: space.y };
            bestSpace = space;
            bestRotation = true;
            break;  // Use o primeiro espaço adequado encontrado
                }
        }

        if (bestFit) {
            this.placePieceInSpace(sheet, bestSpace, piece, margin, bestRotation);
            return true;
        }

        return false;
    }

    fitsInSpace(space, width, height) {
        return space.width >= width && space.height >= height;
    }

    placePieceInSpace(sheet, space, piece, margin, rotated) {
        const pieceWidth = rotated ? piece.height : piece.width;
        const pieceHeight = rotated ? piece.width : piece.height;

        const effectiveWidth = pieceWidth + margin * 2 + this.sawKerf;
        const effectiveHeight = pieceHeight + margin * 2 + this.sawKerf;

        sheet.placements.push({
            x: space.x + margin,
            y: space.y + margin,
            width: pieceWidth,
            height: pieceHeight,
            name: piece.name,
            rotated: rotated
        });

        this.updateSpaces(sheet, space, effectiveWidth, effectiveHeight);
    }

    updateSpaces(sheet, usedSpace, width, height) {
        const spaceIndex = sheet.spaces.indexOf(usedSpace);
        if (spaceIndex !== -1) {
            sheet.spaces.splice(spaceIndex, 1);
        }

        const newSpaces = [];

        if (usedSpace.width - width > this.sawKerf * 2) {
            newSpaces.push({
                x: usedSpace.x + width,
                y: usedSpace.y,
                width: usedSpace.width - width,
                height: usedSpace.height
            });
        }

        if (usedSpace.height - height > this.sawKerf * 2) {
            newSpaces.push({
                x: usedSpace.x,
                y: usedSpace.y + height,
                width: width,
                height: usedSpace.height - height
            });

            if (usedSpace.width - width > this.sawKerf * 2) {
                newSpaces.push({
                    x: usedSpace.x + width,
                    y: usedSpace.y + height,
                    width: usedSpace.width - width,
                    height: usedSpace.height - height
                });
            }
        }

        sheet.spaces.push(...newSpaces);
        this.cleanupSpaces(sheet);
    }

    cleanupSpaces(sheet) {
        sheet.spaces = sheet.spaces.filter(space =>
        space.width > this.sawKerf * 2 &&
        space.height > this.sawKerf * 2
        );

        for (let i = sheet.spaces.length - 1; i >= 0; i--) {
            const space1 = sheet.spaces[i];
            for (let j = 0; j < sheet.spaces.length; j++) {
                if (i === j) continue;
                const space2 = sheet.spaces[j];
                if (this.isSpaceContained(space1, space2)) {
                    sheet.spaces.splice(i, 1);
                    break;
                }
            }
        }

        this.mergeAdjacentSpaces(sheet);
    }

    isSpaceContained(space1, space2) {
        return space1.x >= space2.x &&
        space1.y >= space2.y &&
        (space1.x + space1.width) <= (space2.x + space2.width) &&
        (space1.y + space1.height) <= (space2.y + space2.height);
    }

    isOverlapping(rect1, rect2) {
        return !(rect1.x + rect1.width <= rect2.x ||
        rect2.x + rect2.width <= rect1.x ||
        rect1.y + rect1.height <= rect2.y ||
        rect2.y + rect2.height <= rect1.y);
    }

    mergeAdjacentSpaces(sheet) {
        let merged;
        do {
            merged = false;
            for (let i = 0; i < sheet.spaces.length; i++) {
                for (let j = i + 1; j < sheet.spaces.length; j++) {
                    const space1 = sheet.spaces[i];
                    const space2 = sheet.spaces[j];

                    if (this.canMergeHorizontally(space1, space2)) {
                        space1.width = space2.x + space2.width - space1.x;
                        sheet.spaces.splice(j, 1);
                        merged = true;
                        break;
                    }
                    else if (this.canMergeVertically(space1, space2)) {
                        space1.height = space2.y + space2.height - space1.y;
                        sheet.spaces.splice(j, 1);
                        merged = true;
                        break;
                    }
                }
                if (merged) break;
            }
        } while (merged);
    }

    canMergeHorizontally(space1, space2) {
        return space1.y === space2.y &&
        space1.height === space2.height &&
        Math.abs(space1.x + space1.width - space2.x) < this.sawKerf;
    }

    canMergeVertically(space1, space2) {
        return space1.x === space2.x &&
        space1.width === space2.width &&
        Math.abs(space1.y + space1.height - space2.y) < this.sawKerf;
    }

    renderCuttingPlan() {
        const container = document.getElementById('cuttingPlanContainer');
        if (!container) return;

        container.innerHTML = '';

        Object.entries(this.sheets).forEach(([material, sheets]) => {
            if (!sheets || !sheets.length) return;

            const materialSection = document.createElement('div');
            materialSection.className = 'material-section';

            const materialHeader = document.createElement('h3');
            materialHeader.textContent = `Material: ${material}`;
            materialSection.appendChild(materialHeader);

            sheets.forEach((sheet, sheetIndex) => {
                const sheetWrapper = this.createSheetWrapper(material, sheet, sheetIndex);
                if (sheetWrapper) {
                    materialSection.appendChild(sheetWrapper);
                }
            });

            container.appendChild(materialSection);
        });
    }

    createSheetWrapper(material, sheet, sheetIndex) {
        if (!sheet) return null;

        const sheetWrapper = document.createElement('div');
        sheetWrapper.className = 'sheet-wrapper';

        const sheetDiv = document.createElement('div');
        sheetDiv.className = 'sheet';
        sheetDiv.style.width = `${this.sheetWidth * this.scale}px`;
        sheetDiv.style.height = `${this.sheetHeight * this.scale}px`;

        // Adiciona título da chapa
        const sheetLabel = document.createElement('div');
        sheetLabel.className = 'sheet-label';
        sheetLabel.textContent = `${material} - Chapa ${sheetIndex + 1}`;
        sheetWrapper.appendChild(sheetLabel);

        // Renderiza os espaços disponíveis
        this.renderAvailableSpaces(sheet, sheetDiv);

        // Renderiza as peças
        if (sheet.placements) {
            sheet.placements.forEach(piece => this.renderPiece(piece, sheetDiv));
        }

        sheetWrapper.appendChild(sheetDiv);
        return sheetWrapper;
    }


    renderAvailableSpaces(sheet, sheetDiv) {
        const validSpaces = sheet.spaces.filter(space => {
            return !sheet.placements.some(placement =>
            this.isOverlapping(space, {
                x: placement.x - this.sawKerf,
                y: placement.y - this.sawKerf,
                width: placement.width + this.sawKerf * 2,
                height: placement.height + this.sawKerf * 2
            })
            );
        });

        validSpaces.forEach(space => {
            if (space.width > this.sawKerf * 3 && space.height > this.sawKerf * 3) {
                const spaceDiv = document.createElement('div');
                spaceDiv.className = 'available-space';
                spaceDiv.style.left = `${space.x * this.scale}px`;
                spaceDiv.style.top = `${space.y * this.scale}px`;
                spaceDiv.style.width = `${space.width * this.scale}px`;
                spaceDiv.style.height = `${space.height * this.scale}px`;

                const spaceLabel = document.createElement('div');
                spaceLabel.className = 'space-label';
                spaceLabel.textContent = `${Math.round(space.width)} × ${Math.round(space.height)}mm`;
                spaceDiv.appendChild(spaceLabel);

                sheetDiv.appendChild(spaceDiv);
            }
        });
    }

    renderPiece(piece, sheetDiv) {
        const pieceDiv = document.createElement('div');
        pieceDiv.className = 'piece';
        if (piece.rotated) pieceDiv.classList.add('rotated');

        pieceDiv.style.left = `${piece.x * this.scale}px`;
        pieceDiv.style.top = `${piece.y * this.scale}px`;
        pieceDiv.style.width = `${piece.width * this.scale}px`;
        pieceDiv.style.height = `${piece.height * this.scale}px`;

        // Adiciona o nome da peça
        const nameLabel = document.createElement('div');
        nameLabel.className = 'piece-name-label';
        nameLabel.textContent = piece.name;

        // Determina se o texto deve ser rotacionado baseado nas dimensões
        const isVertical = piece.height > piece.width;
        if (isVertical) {
            nameLabel.style.transform = 'rotate(-90deg)';
            nameLabel.style.width = `${piece.height * this.scale}px`;
            nameLabel.style.maxWidth = `${piece.height * this.scale}px`;
        } else {
            nameLabel.style.width = `${piece.width * this.scale}px`;
            nameLabel.style.maxWidth = `${piece.width * this.scale}px`;
        }

        // Adiciona as dimensões
        const widthLabel = document.createElement('div');
        widthLabel.className = 'dimension-label width-label';
        widthLabel.textContent = `${Math.round(piece.width)}`;

        const heightLabel = document.createElement('div');
        heightLabel.className = 'dimension-label height-label';
        heightLabel.textContent = `${Math.round(piece.height)}`;

        pieceDiv.appendChild(nameLabel);
        pieceDiv.appendChild(widthLabel);
        pieceDiv.appendChild(heightLabel);

        pieceDiv.title = `${piece.name}\nLargura: ${piece.width}mm\nAltura: ${piece.height}mm${piece.rotated ? '\n(Rotacionado)' : ''}`;

        sheetDiv.appendChild(pieceDiv);
    }

    addDimensionLabels(pieceDiv, piece) {
        const widthLabel = document.createElement('div');
        widthLabel.className = 'dimension-label width-label';
        widthLabel.textContent = `${Math.round(piece.width)}`;
        pieceDiv.appendChild(widthLabel);

        const heightLabel = document.createElement('div');
        heightLabel.className = 'dimension-label height-label';
        heightLabel.textContent = `${Math.round(piece.height)}`;
        pieceDiv.appendChild(heightLabel);
    }

    updateStatistics() {
        const stats = this.calculateStatistics();

        const sheetCountEl = document.getElementById('sheetCount');
        const usedAreaEl = document.getElementById('usedArea');
        const wasteAreaEl = document.getElementById('wasteArea');

        if (sheetCountEl) sheetCountEl.textContent = stats.totalSheets;
        if (usedAreaEl) usedAreaEl.textContent = `${stats.usedPercentage}%`;
        if (wasteAreaEl) wasteAreaEl.textContent = `${stats.wastePercentage}%`;
    }

    calculateStatistics() {
        let totalSheets = 0;
        let totalArea = 0;
        let usedArea = 0;

        Object.values(this.sheets).forEach(materialSheets => {
            if (!materialSheets) return;

            totalSheets += materialSheets.length;
            totalArea += materialSheets.length * this.sheetWidth * this.sheetHeight;

            materialSheets.forEach(sheet => {
                if (!sheet.placements) return;

                sheet.placements.forEach(placement => {
                    if (placement.width && placement.height) {
                        usedArea += placement.width * placement.height;
                    }
                });
            });
        });

        const usedPercentage = totalArea ? ((usedArea / totalArea) * 100).toFixed(1) : "0.0";
        const wastePercentage = (100 - parseFloat(usedPercentage)).toFixed(1);

        return {
            totalSheets,
            usedPercentage,
            wastePercentage
        };
    }


    exportToCSV() {
        let csv = 'Material,Chapa,Nome da Peça,Largura (mm),Altura (mm),Rotacionado\n';

        Object.entries(this.sheets).forEach(([material, sheets]) => {
            sheets.forEach((sheet, sheetIndex) => {
                sheet.placements.forEach(placement => {
                    csv += `${material},${sheetIndex + 1},${placement.name},${placement.width},${placement.height},${placement.rotated ? 'Sim' : 'Não'}\n`;
                });
            });
        });

        const blob = new Blob([csv], { type: 'text/csv;charset=utf-8;' });
        const link = document.createElement('a');
        link.href = URL.createObjectURL(blob);
        link.download = `plano_de_corte_${new Date().toISOString().slice(0,10)}.csv`;
        link.click();
    }



  //  generatePDF() {
        // Implementação futura usando jsPDF ou similar
   //     alert('Funcionalidade de exportação PDF em desenvolvimento');
   // }

    toggleAllPieces(event) {
        const checkboxes = document.querySelectorAll('.piece-selector');
        checkboxes.forEach(checkbox => {
            checkbox.checked = event.target.checked;
        });
        this.updatePieceSelection();
    }

    updatePieceSelection() {
        const anySelected = Array.from(document.querySelectorAll('.piece-selector'))
        .some(checkbox => checkbox.checked);
        document.getElementById('calculate').disabled = !anySelected;
    }
}

// Inicialização quando o documento estiver carregado
document.addEventListener('DOMContentLoaded', () => {
    window.cuttingPlan = new CuttingPlan();
    window.cuttingPlan.setupBudgetEvents();


    // Adiciona listeners para inputs que afetam o plano de corte
    const recalculateTriggers = ['margin', 'allowRotation', 'respectGrain'];
    recalculateTriggers.forEach(id => {
        const element = document.getElementById(id);
        if (element) {
            element.addEventListener('change', () => {
                if (document.querySelectorAll('.piece-selector:checked').length > 0) {
                    window.cuttingPlan.generateCuttingPlan();
                }
            });
        }
    });
    window.cuttingPlan.calculateBudget();


    // Adiciona tooltips e interatividade
    document.querySelectorAll('.setting-group label').forEach(label => {
        const input = label.querySelector('input');
        if (input) {
            switch(input.id) {
                case 'allowRotation':
                    label.title = 'Permite que as peças sejam rotacionadas automaticamente para melhor aproveitamento';
                    break;
                case 'respectGrain':
                    label.title = 'Mantém a orientação das peças para respeitar a direção dos veios da madeira';
                    break;
                case 'margin':
                    label.title = 'Espaço adicional ao redor de cada peça para compensar imprecisões no corte';
                    break;
                case 'sawKerf':
                    label.title = 'Espessura da lâmina da serra que será perdida durante o corte';
                    break;
            }
        }
    });

    // Atualiza data e usuário
    if (document.getElementById('currentDate')) {
        document.getElementById('currentDate').textContent = '2025-03-15 19:53:42';
    }
    if (document.getElementById('currentUser')) {
        document.getElementById('currentUser').textContent = 'teneir';
    }
});

// Função para comunicação com o SketchUp
function closeDialog() {
    window.location = 'skp:closeDialog@';
}
