require 'sketchup.rb'
require 'moveis3d/tewe_plugin.rb'
#require 'moveis3d/reporter.rb'
require 'moveis3d/criar_componente.rb' # Nova ferramenta
require 'moveis3d/rotacionar.rb'
require 'moveis3d/edit_component.rb'
require 'moveis3d/list_components.rb' # 🔄 Substituído scale_from_point.rb por list_components.rb


module Moveis3D
    PLUGIN_NAME = "Moveis3D"
    PLUGIN_VERSION = "1.0"
    PLUGIN_AUTHOR = "Teneir Wenginowicz"
    PLUGIN_DESCRIPTION = "Plugin para SketchUp com ferramentas para móveis."
    PLUGIN_LICENSE = "Gratuito"
    PLUGIN_URL = "https://moveis3d.com"
end


module Moveis3D
    class Main
        def self.start
            # Evita carregar o menu mais de uma vez
            unless @menu_loaded
                @menu_loaded = true

                # 📌 Criando o menu do plugin
                menu = UI.menu('Plugins').add_submenu(Moveis3D::PLUGIN_NAME)

                # 🎨 Criando a barra de ferramentas
                toolbar = UI::Toolbar.new(Moveis3D::PLUGIN_NAME)

                # 🔹 Comando: Modelos 3D (Abre o diálogo)
                cmd_modelos_3d = UI::Command.new("Modelos 3D") do
                    Moveis3D::Plugin.show_dialog
                end
                cmd_modelos_3d.small_icon = File.join(File.dirname(__FILE__), 'modelos_small.png')
                cmd_modelos_3d.large_icon = File.join(File.dirname(__FILE__), 'modelos_large.png')
                cmd_modelos_3d.tooltip = "Abrir Modelos 3D"
                menu.add_item(cmd_modelos_3d)  # Adiciona ao menu
                toolbar.add_item(cmd_modelos_3d)  # Adiciona à barra

                # 🔹 Comando: Criar Componente 18mm
                cmd_criar_componente = UI::Command.new("Criar Componente 18mm") do
                    Sketchup.active_model.select_tool(Moveis3D::Tools::CriarComponenteTool.new)
                end
                cmd_criar_componente.small_icon = File.join(File.dirname(__FILE__), 'criar_small.png')
                cmd_criar_componente.large_icon = File.join(File.dirname(__FILE__), 'criar_large.png')
                cmd_criar_componente.tooltip = "Criar um Componente de 18mm"
                menu.add_item(cmd_criar_componente)
                toolbar.add_item(cmd_criar_componente)

                # 🔹 Comando: rotacionar
                cmd_rotacionar = UI::Command.new("rotacionar componente") do
                    Sketchup.active_model.select_tool(Moveis3D::RotateFurnitureTool.new)
                end
                cmd_rotacionar.small_icon = File.join(File.dirname(__FILE__), 'rotate_large.png')
                cmd_rotacionar.large_icon = File.join(File.dirname(__FILE__), 'rotate_large.png')
                cmd_rotacionar.tooltip = "Rotacionar componente em 90º"
                menu.add_item(cmd_rotacionar)
                toolbar.add_item(cmd_rotacionar)

                # 🔹 Comando: Plano de Corte
              #  cmd_plano_corte = UI::Command.new("Plano de Corte") do
              #      Moveis3D.cutlist_interactive_menu
              #  end
              #  cmd_plano_corte.small_icon = File.join(File.dirname(__FILE__), 'plano_small.png')
              #  cmd_plano_corte.large_icon = File.join(File.dirname(__FILE__), 'plano_large.png')
              #  cmd_plano_corte.tooltip = "Abrir Plano de Corte"
              #  menu.add_item(cmd_plano_corte)
              #  toolbar.add_item(cmd_plano_corte)

                # 🔹 Comando: Listar Componentes (Nova ferramenta)
                cmd_list_components = UI::Command.new("Listar Componentes") do
                    Moveis3D.show_component_list
                end
                cmd_list_components.small_icon = File.join(File.dirname(__FILE__), 'plano_small.png')
                cmd_list_components.large_icon = File.join(File.dirname(__FILE__), 'plano_large.png')
                cmd_list_components.tooltip = "Mostrar lista de componentes"
                menu.add_item(cmd_list_components)
                toolbar.add_item(cmd_list_components)

                # Exibir a barra de ferramentas
                toolbar.show
            end
        end
    end

    #def self.cutlist_interactive_menu
    #    cutlist_webGui = SteveR::CutList::WebGui.new("")
    #    cutlist_webGui.start
    #end
end

# Inicia o plugin automaticamente ao carregar
Moveis3D::Main.start
