module Moveis3D
  class RotateFurnitureTool
    def activate
      @input_point = Sketchup::InputPoint.new
      @direction = :xy
      @mouse_position = nil
      update_cursor
    end

    def onMouseMove(flags, x, y, view)
      @mouse_position = [x, y]
      @input_point.pick(view, x, y)
      view.tooltip = @input_point.tooltip if @input_point.valid?

      update_rotation_direction(view, x, y) unless @locked_direction
      update_cursor
      view.invalidate
    end

    def onLButtonDown(flags, x, y, view)
      @input_point.pick(view, x, y)
      picked_entity = Sketchup.active_model.selection.first

      if picked_entity.is_a?(Sketchup::ComponentInstance) || picked_entity.is_a?(Sketchup::Group)
        rotate_entity(picked_entity, @input_point.position)
      else
        UI.messagebox("Selecione um componente ou grupo para girar.")
      end
    end

    def draw(view)
      return unless @mouse_position

      point3d = view.inputpoint(@mouse_position[0], @mouse_position[1]).position

      case @direction
      when :xy
        normal = Geom::Vector3d.new(0, 0, 1)  # 🔵 Rotação no eixo Z (XY)
        view.drawing_color = "blue"
      #  puts "Desenhando no eixo XY (azul)"
      when :xz
        normal = Geom::Vector3d.new(0, 1, 0)  # 🔴 Rotação no eixo Y (XZ)
        view.drawing_color = "red"
       # puts "Desenhando no eixo XZ (vermelho)"
      when :yz
        normal = Geom::Vector3d.new(2, 1, 0)  # 🟢 Rotação no eixo X (YZ)
        view.drawing_color = "green"
       # puts "Desenhando no eixo YZ (verde)"
      else
        puts "Direção desconhecida: #{@direction}"
      end

      view.line_width = 5 # Aumenta a espessura das linhas
      draw_circle(view, point3d, 30, normal)
    end

    def draw_circle(view, center, radius, normal)
      num_segments = 36
      angle_step = 360.degrees / num_segments
      points = []

      num_segments.times do |i|
        angle = i * angle_step
        transformation = Geom::Transformation.rotation(center, normal, angle)
        point = center + (Geom::Vector3d.new(radius, 0, 0).transform(transformation))
        points << point
      end

      points << points.first # Fecha o círculo
      view.draw(GL_LINE_LOOP, points) # Desenha apenas o contorno
    end

    def update_rotation_direction(view, x, y)
      picked_entity = Sketchup.active_model.selection.first
      return unless picked_entity

      axis = determine_rotation_axis(@input_point.position, picked_entity.bounds)

      case axis
      when Geom::Vector3d.new(0, 0, 1)
        @direction = :xy
      when Geom::Vector3d.new(1, 0, 0)
        @direction = :yz
      when Geom::Vector3d.new(0, 1, 0)
        @direction = :xz
      else
        puts "Eixo de rotação não reconhecido: #{axis}"
      end
    end

    def update_cursor
      cursor_path = case @direction
      when :xy
        File.join(__dir__, 'rot_azul.png')
      when :xz
        File.join(__dir__, 'rot_vermelho.png')
      when :yz
        File.join(__dir__, 'rot_verde.png')
      end
      @cursor_id = UI.create_cursor(cursor_path, 0, 0)
      UI.set_cursor(@cursor_id)
    end

    def onSetCursor
      UI.set_cursor(@cursor_id)
    end

    def onKeyDown(key, repeat, flags, view)
      @locked_direction = true
      case key
      when VK_LEFT
        @direction = :yz
      when VK_RIGHT
        @direction = :xz
      when VK_UP
        @direction = :xy
      end
      update_cursor
    end

    def rotate_entity(entity, click_point)
      model = Sketchup.active_model
      model.start_operation('Rotacionar Peça', true)

      bounds = entity.bounds
      center = bounds.center

      axis = case @direction
      when :xy then Geom::Vector3d.new(0, 0, 1)
      when :xz then Geom::Vector3d.new(0, 1, 0)
      when :yz then Geom::Vector3d.new(1, 0, 0)
      end

      transformation = Geom::Transformation.rotation(center, axis, 90.degrees)
      entity.transform!(transformation)

      model.commit_operation
    rescue => e
      UI.messagebox("Erro ao rotacionar a peça: #{e.message}")
      model.abort_operation
    end

    private

    def determine_rotation_axis(click_point, bounds)
      center = bounds.center
      x_dist = (click_point.x - center.x).abs
      y_dist = (click_point.y - center.y).abs
      z_dist = (click_point.z - center.z).abs

      if x_dist > y_dist && x_dist > z_dist
        Geom::Vector3d.new(1, 0, 0) # 🟢 Eixo X
      elsif y_dist > x_dist && y_dist > z_dist
        Geom::Vector3d.new(0, 1, 0) # 🔴 Eixo Y
      else
        Geom::Vector3d.new(0, 0, 1) # 🔵 Eixo Z
      end
    end
  end
end
